'*****************************************************************************************************
' monitorLightSource.vbs - monitor a light source
'
' Copyright Hilary Jones, 2010
' hdjones@pacbell.net
'
' This script monitors a light source to see how it and/or the CCD camera change as a function
' of time.  Then intent is to watch the camera while it is warming up to see when the output has
' reached a steady state; but it could also be viewed as tool for monitoring whether the light
' source changes with time.
'
' Future:
'	Make Excel window move to the foreground
'	Optionally save the ADU values in a file for later study
'	Bug: if user clicks on a cell in the spreadsheet, we get an exception
'		Currently I'm making that cause the script to exit, but it would be better to prevent the problem
'	Get rid of Excel's menu items and other controls.  We just want to see the chart.
'	Make sure workbook fills window; and preferably have chart fit window w/o extra space
'	Put data on one sheet and graph on another (arguable)
'
'*****************************************************************************************************

Option Explicit

'*****************************************************************************************************
' Parameters that may need to be changed for each run:
'	E			exposure time
'	filterNumber		the filter number to use (needs to be changed for different cameras!)
'	numExtras		number of extra exposures between chart points (to keep chart from being cluttered)
'	size			size of the subframe (if any)
'	usingSubFrames		True => use subframes
'
'*****************************************************************************************************

' Define the exposure time in seconds
Const E = 5

' Define the filter to be used.  This is a slot number, and may change for different cameras!
Const filterNumber = 3	' 3 => Luminosity for the ST-10

' Define the number of extra exposures between chart points.  Use 0 to record every exposure on the chart.
Const numExtras = 0

' Define the size of the subframe (if any)
Const size = 400

' Define whether to use subframes
Const usingSubFrames = True


'*****************************************************************************************************
' Main part of program
'*****************************************************************************************************

Dim I			' General purpose loop variable
Dim range		' Used to define where to store data and how big chart is
Dim rowData(1)		' One row of data for the chart
Dim startTime		' Time when the test started

' Set up the camera
Dim camera
Set camera = CreateObject("MaxIm.CCDCamera")
camera.LinkEnabled = True
camera.DisableAutoShutdown = True			' Leave camera on when we are done

if Not camera.LinkEnabled Then
   wscript.echo "Failed to start camera."
   Quit
End If

' If using subframes, set up location and size
If (usingSubFrames) Then
	Dim left, top
	left = (camera.CameraXSize - size)/2
	top = (camera.CameraYSize - size)/2
	camera.NumX = size
	camera.NumY = size
	camera.StartX = left
	camera.StartY = top
End If



' Set up Excel
Dim excel
Set excel = Createobject("Excel.Application")
excel.DisplayAlerts = False	' We don't want to be prompted to save the data when we are done
Dim workbook
set workbook = excel.Workbooks.Add()

' Set up titles for the data table
rowData(0) = "Time"
rowData(1) = "ADU"
set range = workbook.Worksheets("Sheet1").Range("A1:B1")
range.value = rowData

' Create a chart, initially empty
workbook.Charts.Add
workbook.ActiveChart.ChartType = -4169  ' xlXYScatter
workbook.ActiveChart.HasTitle = True
workbook.ActiveChart.ChartTitle.Text = "Brightness variation"
workbook.ActiveChart.Axes(1).HasTitle = True
workbook.ActiveChart.Axes(1).AxisTitle.Text = "Time (seconds)"
workbook.ActiveChart.Axes(2).HasTitle = True
workbook.ActiveChart.Axes(2).AxisTitle.Text = "ADU"
workbook.ActiveChart.Legend.Delete()
workbook.ActiveChart.Location 2, "Sheet1" ' xlLocationAsObject => will be an object in sheet 1


' Note: if I use set "excel.Interactive = False", the user can't upset this script by trying to
' interact with Excel; but he also can't use the close box to terminate the script.  The
' following statement doesn't help; but it does ensure that Excel quits if the script quits.
excel.UserControl = False

excel.Visible = True


' Start acquiring data
Dim ok
ok = MsgBox("This script will run forever until the Excel spreadsheet is closed.  Press OK to start.", VBOKCancel)
If (ok = VBCancel) Then
	excel.Quit
	wscript.Quit
End If

On Error Resume Next	' All errors cause script to exit
startTime = Timer()
Dim row
row = 1			' First row (the header)
Do While True
	row = row + 1
	Dim time
	time = Timer() - startTime
	Dim value
	value = ADU(E)
	call addPoint(row, time, value)
	If Err.Number <> 0 Then
		workBook.Close()
		wscript.Quit()
	End If
	For I = 1 To numExtras
		' Keep camera busy taking exposures, but don't record them in the chart
		value = ADU(E)
	Next
Loop

'************************************************************************
' A function for taking an image with a given exposure time, and finding the average ADU value
Function ADU(exposureTime)
	camera.Expose exposureTime, 1, filterNumber' exposure time, shutter open, filter
	Do While Not camera.ImageReady
		' Don't consume CPU time while waiting
		wscript.sleep 100
	Loop
	
	' Find average value
	Dim doc
	Set doc = camera.Document
	' CalcAreaInfo returns variant array whose second element is the average
	ADU = doc.CalcAreaInfo(0, 0, doc.XSize-1, doc.YSize-1)(2)
End Function

'*****************************************************************************************************
' Subroutine to add one data point to the chart
Sub addPoint(rowNumber, time, ADU)
	rowData(0) = time
	rowData(1) = ADU
	Dim rangeString

	' Use range to define where to place the new value
	rangeString = "A" & rowNumber & ":B" & rowNumber
	set range = workbook.Worksheets("Sheet1").Range(rangeString)
	range.value = rowData

	' Now use range to enlarge the chart size
	rangeString = "A1:B" & rowNumber
	set range = workbook.Worksheets("Sheet1").Range(rangeString)
	workbook.ActiveChart.SetSourceData range, 2 ' xlColumns  [???]
End Sub
